Package: blisa
Type: Package
Title: Infer Cell-Cell Communication from Spatial Transcriptomics
Version: 0.2.0
Authors@R: c(
    person(given = "Yunshun", family = "Chen",
           role = c("aut", "cre"),
           email = "yuchen@wehi.edu.au"),
    person(given = "Lei", family = "Qin",
           role = "aut",
           email = "qin.l@wehi.edu.au"),
    person(given = "Lizhong", family = "Chen",
           role = "aut"))
Description: Identifies cell-cell communication hotspots in spatial
    transcriptomics data using bivariate Local Moran's I statistics
    on hexagonally binned cells. Provides functions for spatial
    weighting, ligand-receptor pair filtering, hotspot detection,
    and visualisation of sender-receiver cell-type interactions.
License: GPL (>= 3)
Encoding: UTF-8
Imports: sf, spdep, Matrix, SpatialExperiment, SummarizedExperiment,
        ComplexHeatmap, ggplot2, viridisLite, grid
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Suggests: knitr, rmarkdown
NeedsCompilation: no
Packaged: 2026-05-29 02:55:39 UTC; yuchen
Author: Yunshun Chen [aut, cre],
  Lei Qin [aut],
  Lizhong Chen [aut]
Maintainer: Yunshun Chen <yuchen@wehi.edu.au>
Repository: CRAN
Date/Publication: 2026-06-02 07:50:36 UTC
Built: R 4.5.2; ; 2026-06-02 11:04:40 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(blisa,SpatialExperiment)
S3method(blisa,default)
S3method(plotCCI,blisa)
S3method(plotCCI,default)
S3method(plotCCILR,blisa)
S3method(plotCCILR,default)
S3method(plotCCIsummary,blisa)
S3method(plotCCIsummary,default)
S3method(plotHotspots,blisa)
S3method(plotLRrank,blisa)
S3method(plotLRrank,data.frame)
S3method(print,blisa)
S3method(runCCI,blisa)
S3method(runCCI,default)
export(blisa)
export(computeSpatialWeights)
export(filterLRpairs)
export(hexBinCells)
export(is.blisa)
export(plotCCI)
export(plotCCILR)
export(plotCCIspatial)
export(plotCCIsummary)
export(plotHotspots)
export(plotLRrank)
export(runCCI)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,draw)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(grDevices,colorRampPalette)
importFrom(grid,gpar)
importFrom(grid,unit)
importFrom(stats,setNames)
importFrom(utils,download.file)

# blisa: Cell-cell communication using Bivariate Local Indicator of Spatial Autocorrelation

*blisa* implements functionalities to study cell-cell communication in spatial transcriptomics experiments using bivariate local indicator of spatial autocorrelation.

Install development version from GitHub
```
library(devtools)   
devtools::install_github("ChenLaboratory/blisa")
```

#' Compute Spatial Weights for BLISA
#'
#' Builds queen (nearby) and distance-decay (diffuse) spatial weight matrices
#' from a bin-level \code{sf} object, excluding isolated bins and optionally
#' excluding low-cell bins. A second-pass isolation check further removes bins
#' that become isolated after the initial subset.
#'
#' @param bins An \code{sf} object of spatial bins.
#' @param bin_size Numeric. Bin spacing used to define queen adjacency
#'   (\code{1.2 * bin_size} radius).
#' @param dmax Numeric. Maximum distance for diffuse-mode neighbours.
#' @param min_cells Integer. Minimum cell count for a bin to be
#'   included. Ignored when \code{n_cells_col = NA}.
#' @param n_cells_col Character or \code{NA}. Column name in \code{bins}
#'   holding per-bin cell counts. Set to \code{NA} to skip cell-count
#'   filtering (default).
#'
#' @return A list with:
#' \describe{
#'   \item{queen_wt}{Spatial weights list for nearby (queen) mode.}
#'   \item{dist_wt}{Spatial weights list for diffuse (distance-decay) mode.}
#'   \item{keep_idx_queen}{Integer indices of bins used in queen-mode Moran.}
#'   \item{keep_idx_dist}{Integer indices of bins used in diffuse-mode Moran.}
#'   \item{isolate_idx_queen}{Integer indices of original queen-mode isolates.}
#'   \item{isolate_idx_dist}{Integer indices of original diffuse-mode isolates.}
#'   \item{low_cell_idx}{Integer indices of bins excluded for low cell counts.}
#'   \item{queen_nb_full}{Full (unsubset) neighbour list for nearby mode, indexed over all bins.}
#'   \item{dist_nb_full}{Full (unsubset) neighbour list for diffuse mode, indexed over all bins.}
#' }
#' @export
computeSpatialWeights <- function(bins,
                                  bin_size    = 50,
                                  dmax        = 250,
                                  min_cells   = 1,
                                  n_cells_col = NA) {
  centroids <- sf::st_centroid(bins)
  coords    <- sf::st_coordinates(centroids)
  n_bins    <- nrow(bins)

  ## ---------------------------
  ## Filter low-cell bins
  ## ---------------------------
  if (!is.na(n_cells_col)) {
    if (!n_cells_col %in% colnames(bins))
      stop("Column '", n_cells_col, "' not found in bins.")
    low_cell_idx <- which(bins[[n_cells_col]] < min_cells)
    message(length(low_cell_idx), " bins removed: < ", min_cells,
            " cells (column: '", n_cells_col, "').")
  } else {
    low_cell_idx <- integer(0)
    message("n_cells_col = NA \u2014 cell-count filtering skipped.")
  }

  ## ---------------------------
  ## Helper: second-pass isolation check
  ## After subset.nb, bins whose neighbours were all excluded become new isolates.
  ## Detect them and further shrink keep_idx before building weights.
  ## ---------------------------
  resolve_new_isolates <- function(nb, nb_full, keep_idx) {
    new_iso_sub <- which(spdep::card(nb) == 0) # positions in subset space
    if (length(new_iso_sub) == 0) return(list(nb = nb, keep_idx = keep_idx))
    new_iso_full <- keep_idx[new_iso_sub] # back to full-space indices
    message(length(new_iso_full), " bins became isolated after subset \u2014 excluded.")
    keep_idx <- setdiff(keep_idx, new_iso_full)
    nb       <- spdep::subset.nb(nb_full, subset = seq_len(n_bins) %in% keep_idx)
    list(nb = nb, keep_idx = keep_idx)
  }

  ## ---------------------------
  ## Queen spatial weights  (for "nearby" mode)
  ## ---------------------------
  queen_nb_full     <- spdep::dnearneigh(coords, 0, 1.2 * bin_size)
  isolate_idx_queen <- which(spdep::card(queen_nb_full) == 0)
  message(length(isolate_idx_queen), " isolated bins with no nearby neighbours: ",
          paste(isolate_idx_queen, collapse = ","))

  keep_idx_queen <- setdiff(seq_len(n_bins), union(isolate_idx_queen, low_cell_idx))
  queen_nb       <- spdep::subset.nb(queen_nb_full,
                                     subset = seq_len(n_bins) %in% keep_idx_queen)
  r              <- resolve_new_isolates(queen_nb, queen_nb_full, keep_idx_queen)
  queen_nb       <- r$nb;  keep_idx_queen <- r$keep_idx

  queen_wt <- spdep::nb2listwdist(queen_nb, centroids[keep_idx_queen, ],
                                  type = "idw", style = "W", zero.policy = TRUE)

  ## ---------------------------
  ## Distance spatial weights  (for "diffuse" mode)
  ## ---------------------------
  dist_nb_full     <- spdep::dnearneigh(coords, 0, dmax)
  isolate_idx_dist <- which(spdep::card(dist_nb_full) == 0)
  message(length(isolate_idx_dist), " isolated bins with no neighbours within ",
          dmax, " um: ", paste(isolate_idx_dist, collapse = ","))

  keep_idx_dist <- setdiff(seq_len(n_bins), union(isolate_idx_dist, low_cell_idx))
  dist_nb       <- spdep::subset.nb(dist_nb_full,
                                    subset = seq_len(n_bins) %in% keep_idx_dist)
  r             <- resolve_new_isolates(dist_nb, dist_nb_full, keep_idx_dist)
  dist_nb       <- r$nb;  keep_idx_dist <- r$keep_idx

  weight_at_dmax <- 0.01
  dist_wt <- spdep::nb2listwdist(dist_nb, bins[keep_idx_dist, ],
                                 type = "exp", style = "W", zero.policy = TRUE,
                                 alpha = -log(weight_at_dmax) / dmax)

  list(
    queen_wt          = queen_wt,
    dist_wt           = dist_wt,
    keep_idx_queen    = keep_idx_queen,
    keep_idx_dist     = keep_idx_dist,
    isolate_idx_queen = isolate_idx_queen,
    isolate_idx_dist  = isolate_idx_dist,
    low_cell_idx      = low_cell_idx,
    queen_nb_full     = queen_nb_full,
    dist_nb_full      = dist_nb_full
  )
}

# Internal helper: resolve a row index from optional ligand/receptor args.
# Returns `index` unchanged when neither ligand nor receptor is supplied.
.resolve_lr_index <- function(LR_results, index, ligand, receptor) {
  if (!is.null(ligand) && !is.null(receptor)) {
    matches <- which(LR_results$ligand.symbol == ligand &
                       LR_results$receptor.symbol == receptor)
    if (length(matches) == 0)
      stop("No LR pair found with ligand = '", ligand,
           "' and receptor = '", receptor, "'.")
    if (length(matches) > 1)
      warning("Multiple rows match ligand = '", ligand, "' and receptor = '",
              receptor, "'. Using the first match (index ", matches[1], ").")
    return(matches[1])
  }
  if (xor(!is.null(ligand), !is.null(receptor)))
    stop("'ligand' and 'receptor' must be provided together.")
  index
}


#' Spatial hotspot map for one ligand-receptor pair
#'
#' Generic function. Plots each bin coloured by significance status: empty,
#' non-significant, or significant hotspot (continuous gradient of
#' -log10 p-value or 1 - p-value).
#'
#' @param x A \code{blisa} object.
#' @param ... Additional arguments passed to the method.
#'
#' @return A \code{ggplot} object.
#' @export
plotHotspots <- function(x, ...) UseMethod("plotHotspots")


#' @describeIn plotHotspots Method for a \code{blisa} object.
#'
#' @param index Integer. Row index into \code{LR_results} selecting the
#'   ligand-receptor pair to visualise. Ignored when both \code{ligand} and
#'   \code{receptor} are supplied. Default \code{1} (top-ranked pair).
#' @param ligand Character. Ligand gene symbol. When both \code{ligand} and
#'   \code{receptor} are provided the matching LR pair is located automatically
#'   and \code{index} is ignored. Must be supplied together with
#'   \code{receptor}.
#' @param receptor Character. Receptor gene symbol. Must be supplied together
#'   with \code{ligand}.
#' @param log_pval Logical. If \code{TRUE} (default), colour significant bins
#'   by -log10(p-value). If \code{FALSE}, use 1 - p-value.
#' @param p_cutoff Numeric or \code{NULL}. When \code{NULL} (default), the
#'   pre-computed hotspot bins stored in the \code{blisa} object are used,
#'   reflecting the \code{p_cutoff} and High-High quadrant classification
#'   applied during \code{\link{blisa}}. When a numeric value is supplied,
#'   bins are re-defined on the fly as those with \code{all_pval <= p_cutoff}
#'   and quadrant label \code{"High-High"} (from the stored
#'   \code{all_quadrant}), giving an exact re-threshold consistent with the
#'   original classification.
#'
#' @export
plotHotspots.blisa <- function(x, index = 1, ligand = NULL, receptor = NULL,
                               log_pval = TRUE, p_cutoff = NULL, ...) {
  bins       <- x$bins
  LR_results <- x$LR_results

  # Resolve which LR pair to plot
  index <- .resolve_lr_index(LR_results, index, ligand, receptor)

  interaction <- rownames(LR_results)[index]

  # Resolve significant bin indices and their p-values
  if (!is.null(p_cutoff)) {
    all_pval      <- LR_results$all_pval[[index]]
    all_quadrant  <- LR_results$all_quadrant[[index]]
    if (is.null(all_pval) || is.null(all_quadrant))
      stop("'all_pval' and 'all_quadrant' must be present in LR_results to ",
           "use 'p_cutoff'. Ensure blisa() was run to completion.")
    sig_indices <- which(all_pval <= p_cutoff & all_quadrant == "High-High")
    p_values    <- all_pval[sig_indices]
  } else {
    sig_indices <- LR_results$sig_index[[index]]
    p_values    <- LR_results$sig_pval[[index]]
  }

  sig_num <- length(sig_indices)

  if (log_pval) {
    plot_vals <- -log10(p_values)
    lgd_title <- "-log10(pval)"
  } else {
    plot_vals <- 1 - p_values
    lgd_title <- "1-pval"
  }

  # Pre-compute a fill colour for every bin
  fill_cols <- rep("#F0F0F0", nrow(bins))    # empty
  fill_cols[bins$n_cells > 0] <- "#D3D3D3"  # non-significant

  if (length(sig_indices) > 0) {
    pval_range <- range(plot_vals)
    if (pval_range[1] == pval_range[2]) pval_range[2] <- pval_range[1] + 1
    pval_norm  <- (plot_vals - pval_range[1]) / diff(pval_range)
    pal        <- colorRampPalette(LRI_spatial_colors)(100)
    fill_cols[sig_indices] <- pal[as.integer(pval_norm * 99) + 1]
  }

  bins$fill_col <- fill_cols

  # Invisible dummy points to drive the continuous colorbar legend
  val_range <- if (length(plot_vals) > 0) range(plot_vals) else c(0, 1)
  bbox <- sf::st_bbox(bins)
  cx   <- unname((bbox["xmin"] + bbox["xmax"]) / 2)
  cy   <- unname((bbox["ymin"] + bbox["ymax"]) / 2)
  dummy <- data.frame(
    x   = cx,
    y   = cy,
    val = seq(val_range[1], val_range[2], length.out = 100)
  )

  title_suffix <- if (!is.null(p_cutoff)) paste0(" (p \u2264 ", p_cutoff, ")") else ""

  p <- ggplot(bins) +
    geom_sf(aes(fill = fill_col), color = NA) +
    scale_fill_identity() +
    geom_point(data = dummy, aes(x = x, y = y, color = val), alpha = 0) +
    scale_color_gradientn(colours = LRI_spatial_colors, name = lgd_title) +
    guides(color = guide_colorbar(title.position = "top")) +
    labs(
      title    = paste0(interaction, " - ", sig_num, " hotspots", title_suffix),
      subtitle = "Grey scale: Light (Empty bins) | Medium (Non-sig bins)"
    ) +
    theme_void() +
    theme(legend.position = "right")

  return(p)
}

filter_genes <- function(gene_column, gene_list) { # function to filter out the whole interaction if missing any L/R subunits
  sapply(strsplit(as.character(gene_column), ", "), function(genes) {
    filtered_genes <- genes[genes %in% gene_list]
    if (length(filtered_genes) != length(genes)) {
      return(NA)
    } else {
      return(paste(filtered_genes, collapse = ", "))
    }
  })
}


getLRpairs <- function(gene_panel, LR_df = NULL, species = c("human", "mouse")){
  if (is.null(LR_df)) LR_df <- .load_cellchat_db(species)

  LR_df$ligand.symbol <- filter_genes(LR_df$ligand.symbol, gene_panel)
  LR_df$receptor.symbol <- filter_genes(LR_df$receptor.symbol, gene_panel)

  LR_included <- LR_df[!(is.na(LR_df$ligand.symbol) | is.na(LR_df$receptor.symbol)), ]

  return(LR_included)
}

# NOTE: use ligand/receptor.symbol as gene name, ligand/receptor columns are bilogical entity names which may not be a gene, e.g. RA-ALDH1A3
#' Filter ligand-receptor pairs by expression threshold
#'
#' Retains only LR pairs where at least one bin/spot has counts at or above
#' \code{min_ligand} for every ligand subunit and \code{min_receptor} for every
#' receptor subunit.
#'
#' @param counts Gene-by-bin count matrix (dense or sparse). Row names must be
#'   gene symbols.
#' @param min_ligand Numeric. Minimum count threshold for ligand genes. At least
#'   one bin must meet or exceed this value. Default 10.
#' @param min_receptor Numeric. Minimum count threshold for receptor genes. At
#'   least one bin must meet or exceed this value. Default 10.
#' @param LR_df Data frame of ligand-receptor pairs with columns
#'   \code{ligand.symbol} and \code{receptor.symbol} (comma-separated gene
#'   symbols for multi-subunit complexes). When \code{NULL}, the CellChatDB for
#'   the chosen \code{species} is downloaded automatically.
#' @param species Character. Which CellChatDB to download when \code{LR_df} is
#'   \code{NULL}. One of \code{"human"} (default) or \code{"mouse"}.
#'
#' @return A subset of \code{LR_df} containing only pairs that pass the
#'   expression thresholds for both ligand and receptor.
#' @export
filterLRpairs <- function(counts, min_ligand = 10, min_receptor = 10,
                          LR_df = NULL, species = c("human", "mouse")) {
  if (is.null(LR_df)) LR_df <- .load_cellchat_db(species)

  filtered_counts_ligand <- counts[Matrix::rowSums(counts >= min_ligand) > 0, ] # at least one spot has more than min_ligand counts
  filtered_counts_receptor <- counts[Matrix::rowSums(counts >= min_receptor) > 0, ]

  LR_filtered_ligand <- getLRpairs(gene_panel = rownames(filtered_counts_ligand), LR_df)
  LR_filtered_receptor <- getLRpairs(gene_panel = rownames(filtered_counts_receptor), LR_df)

  intersect_LR <- intersect(rownames(LR_filtered_ligand), rownames(LR_filtered_receptor))

  LR_filtered <- LR_filtered_ligand[intersect_LR,]

  return(LR_filtered)

}

#' blisa: Bivariate Local Indicator of Spatial Association for spatial transcriptomics
#'
#' @importFrom ComplexHeatmap Heatmap rowAnnotation draw
#' @importFrom grid gpar unit
#' @importFrom grDevices colorRampPalette
#' @importFrom stats setNames
#' @importFrom utils download.file
#' @importFrom ggplot2 ggplot aes geom_sf geom_point scale_fill_manual
#'   scale_fill_identity scale_color_gradientn scale_color_manual
#'   scale_x_continuous scale_y_continuous expansion guides guide_legend guide_colorbar
#'   labs theme_void theme_minimal theme element_text element_rect coord_cartesian
"_PACKAGE"

# Suppress R CMD check notes for ggplot2 aes() variables
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    # ggplot2 aes (plotCCIspatial)
    "cell_pair_plot",
    # ggplot2 aes (plotHotspots)
    "fill_col", "val", "y",
    # ggplot2 aes (plotLRrank)
    "LR_pair", "sig_numbers", "annotation"
  ))
}

LRI_spatial_colors <- c("#FFFFCC", "#FFD700", "#FF7F00", "#D7301F")

col.pDark <- c(
  "#1F77B4", "#FF7F0E", "#2CA02C", "#D62728", "#9467BD",
  "#8C564B", "#E377C2", "#7F7F7F", "#BCBD22", "#17BECF"
)

col.pLight <- c(
  "#AEC7E8", "#FFBB78", "#98DF8A", "#FF9896", "#C5B0D5",
  "#C49C94", "#F7B6D2", "#C7C7C7", "#DBDB8D", "#9EDAE5"
)

col.pMedium <- c(
  "#729ECE", "#FF9E4A", "#67BF5C", "#ED665D", "#AD8BC9",
  "#A8786E", "#ED97CA", "#A2A2A2", "#CDCC5D", "#6DCCDA"
)

cols <- c(col.pDark, col.pLight, col.pMedium)

# Session-level cache so CellChatDB is only downloaded once per R session
.blisa_cache <- new.env(parent = emptyenv())

.load_cellchat_db <- function(species = c("human", "mouse")) {
  species <- match.arg(species)
  key <- paste0("CellChatDB.", species)
  if (!is.null(.blisa_cache[[key]])) return(.blisa_cache[[key]])
  url <- sprintf(
    "https://raw.githubusercontent.com/jinworks/CellChat/main/data/CellChatDB.%s.rda",
    species
  )
  message("Downloading CellChatDB.", species, " from GitHub (once per session)...")
  tmp_file <- tempfile(fileext = ".rda")
  on.exit(unlink(tmp_file), add = TRUE)
  tryCatch({
    download.file(url, tmp_file, mode = "wb", quiet = TRUE)
    tmp <- new.env(parent = emptyenv())
    load(tmp_file, envir = tmp)
    .blisa_cache[[key]] <- tmp[[key]]$interaction
  }, error = function(e) stop(
    "Failed to download CellChatDB.", species, ". ",
    "Check your internet connection or supply LR_df explicitly."
  ))
  .blisa_cache[[key]]
}

parse_units <- function(s) {
  s <- as.character(s)
  s <- gsub("\\s+", "", s)
  unlist(strsplit(s, "[,_]"))
}

get_min_expr <- function(gene_str, hex_gene_counts) {
  genes <- parse_units(gene_str)
  genes <- genes[genes %in% rownames(hex_gene_counts)]
  n <- ncol(hex_gene_counts)
  if (length(genes) == 0) return(rep(0, n))
  if (length(genes) == 1) return(as.numeric(hex_gene_counts[genes, ]))
  mat <- hex_gene_counts[genes, , drop = FALSE]
  apply(mat, 2, min)
}

LR_df_add_mode <- function(LR_df, col = "annotation", default_mode = "diffuse",
                           diffuse_category = c("Secreted Signaling", "Non-protein Signaling")) {
  if (!col %in% colnames(LR_df)) {
    LR_df$ccc_mode <- default_mode
    message(col, " column is missing. Setting ccc_mode as '", default_mode, "' for all.")
    return(LR_df)
  }
  LR_df$ccc_mode <- ifelse(LR_df[[col]] %in% diffuse_category, "diffuse", "nearby")
  message("ccc_mode is 'diffuse' for category: ", paste(diffuse_category, collapse = ", "), "; 'nearby' for others.")
  LR_df
}

#' Sender-by-receiver heatmap of CCI scores for one ligand-receptor pair
#'
#' Generic function. Reshapes the CCI data frame into a receiver-by-sender
#' cell-type matrix for one selected LR pair and draws a clustered heatmap.
#'
#' @param x A \code{blisa} object or a CCI scores data frame (the
#'   \code{CCI_scores} slot of a \code{blisa} object).
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return A \code{Heatmap} object.
#' @seealso \code{\link{plotCCI}} for an overview heatmap across all LR pairs;
#'   \code{\link{plotCCIsummary}} for an aggregated sender-by-receiver heatmap.
#' @export
plotCCILR <- function(x, ...) UseMethod("plotCCILR")


#' @describeIn plotCCILR Method for a \code{blisa} object. The LR pair is
#'   selected by \code{index} (default 1, the top-ranked pair) unless both
#'   \code{ligand} and \code{receptor} are supplied, in which case the matching
#'   row is located automatically and \code{index} is ignored. Stops with an
#'   informative error if \code{CCI_scores} is \code{NULL} or the selected LR
#'   pair has no significant hotspots.
#'
#' @param index Integer. Row index into \code{LR_results} selecting the
#'   ligand-receptor pair to visualise. Ignored when both \code{ligand} and
#'   \code{receptor} are supplied. Default \code{1} (top-ranked pair).
#' @param ligand Character. Ligand gene symbol. When both \code{ligand} and
#'   \code{receptor} are provided the matching LR pair is located automatically
#'   and \code{index} is ignored. Must be supplied together with
#'   \code{receptor}.
#' @param receptor Character. Receptor gene symbol. Must be supplied together
#'   with \code{ligand}.
#'
#' @export
plotCCILR.blisa <- function(x, index = 1, ligand = NULL, receptor = NULL,
                               ...) {
  if (is.null(x$CCI_scores))
    stop("CCI_scores is NULL. Run runCCI() first to compute CCI scores.")

  LR_results <- x$LR_results

  # Resolve which LR pair to plot (reuses helper from plotHotspots.R)
  index  <- .resolve_lr_index(LR_results, index, ligand, receptor)
  lr_pair <- rownames(LR_results)[index]

  if (LR_results$sig_numbers[index] == 0)
    stop("LR pair '", lr_pair, "' has no significant hotspots and therefore ",
         "no CCI scores.")

  if (!lr_pair %in% colnames(x$CCI_scores))
    stop("LR pair '", lr_pair, "' not found in CCI_scores.")

  plotCCILR.default(x$CCI_scores, lr_pair)
}


#' @describeIn plotCCILR Method for a CCI scores data frame (e.g. the
#'   \code{CCI_scores} slot of a \code{blisa} object). The LR pair is selected
#'   by column name via \code{lr_pair}.
#'
#' @param lr_pair Character. Column name in the CCI scores data frame
#'   corresponding to the ligand-receptor pair to visualise
#'   (e.g. \code{"CXCL12_CXCR4"}).
#'
#' @export
plotCCILR.default <- function(x, lr_pair, ...) {
  CCI_df <- x

  if (!lr_pair %in% colnames(CCI_df))
    stop("LR pair '", lr_pair, "' not found in CCI_df.")

  interaction_scores <- CCI_df[[lr_pair]]
  sender   <- CCI_df$Sender
  receiver <- CCI_df$Receiver

  # Build receiver-by-sender matrix
  all_receivers <- unique(receiver)
  all_senders   <- unique(sender)

  interaction_mat <- matrix(NA,
                            nrow = length(all_receivers),
                            ncol = length(all_senders),
                            dimnames = list(all_receivers, all_senders))

  for (i in seq_along(interaction_scores)) {
    interaction_mat[receiver[i], sender[i]] <- interaction_scores[i]
  }

  f1 <- viridisLite::viridis(10)
  p <- Heatmap(interaction_mat,
               name = "Interaction\nScore",
               col = f1,
               cluster_rows = TRUE,
               cluster_columns = TRUE,
               row_title = paste0(lr_pair, " \u2014 Receiver"),
               column_title = paste0(lr_pair, " \u2014 Sender"),
               row_names_gp = gpar(fontsize = 12, fontface = "bold"),
               column_names_gp = gpar(fontsize = 12, fontface = "bold"),
               column_names_rot = 45,
               heatmap_legend_param = list(title_position = "topcenter"))

  return(p)
}

#' Heatmap of CCI scores across all ligand-receptor pairs
#'
#' Generic function. Draws a clustered heatmap (via \code{ComplexHeatmap})
#' with rows as Sender \eqn{\rightarrow} Receiver cell-type pairs and columns
#' as LR pairs. Row annotations colour-code the sender and receiver cell types.
#'
#' @param x A \code{blisa} object or a CCI scores data frame (the
#'   \code{CCI_scores} slot of a \code{blisa} object). The data frame must
#'   contain columns \code{Sender}, \code{Receiver}, and one column per LR pair.
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return Invisibly returns the \code{Heatmap} object.
#' @seealso \code{\link{plotCCILR}} for a sender-by-receiver heatmap of a
#'   single LR pair; \code{\link{plotCCIsummary}} for an aggregated
#'   sender-by-receiver heatmap across all LR pairs.
#' @export
plotCCI <- function(x, ...) UseMethod("plotCCI")


#' @describeIn plotCCI Method for a \code{blisa} object. Extracts
#'   \code{CCI_scores} and delegates to \code{plotCCI.default}. Stops with an
#'   informative error if \code{CCI_scores} is \code{NULL}.
#'
#' @param top_lr Integer or \code{NULL}. Number of top-ranked LR pairs (by
#'   \code{sig_numbers}) to display as columns. LR pairs in \code{CCI_scores}
#'   are already ordered by rank, so this simply takes the first \code{top_lr}
#'   columns. Default \code{20}.
#' @param top_pairs Integer or \code{NULL}. Number of top sender-receiver pairs
#'   to display as rows, ranked by their maximum interaction score across the
#'   displayed LR pairs (after \code{top_lr} is applied). When \code{NULL} all
#'   rows are shown. Default \code{30}.
#' @param sender Character vector or \code{NULL}. If provided, only rows where
#'   \code{Sender} is in this vector are kept. Applied independently of
#'   \code{receiver} (AND logic when both are supplied). Default \code{NULL}
#'   (all senders).
#' @param receiver Character vector or \code{NULL}. If provided, only rows
#'   where \code{Receiver} is in this vector are kept. Applied independently of
#'   \code{sender} (AND logic when both are supplied). Default \code{NULL}
#'   (all receivers).
#' @param colors Named character vector mapping cell-type names to colours,
#'   used for the sender/receiver row annotations. When \code{NULL} (default),
#'   colours are assigned automatically from the package palette. The British
#'   spelling \code{colours} is accepted as an alias.
#' @param colours Alias for \code{colors} (British spelling). Ignored when
#'   \code{colors} is supplied.
#'
#' @export
plotCCI.blisa <- function(x, top_lr = 20, top_pairs = 30,
                          sender = NULL, receiver = NULL,
                          colors = NULL, colours = NULL, ...) {
  if (is.null(x$CCI_scores))
    stop("CCI_scores is NULL. Run runCCI() first to compute CCI scores.")
  if (is.null(colors)) colors <- colours
  plotCCI.default(x$CCI_scores, top_lr = top_lr, top_pairs = top_pairs,
                  sender = sender, receiver = receiver, colors = colors)
}


#' @describeIn plotCCI Method for a CCI scores data frame (e.g. the
#'   \code{CCI_scores} slot of a \code{blisa} object).
#'
#' @export
plotCCI.default <- function(x, top_lr = 20, top_pairs = 30,
                            sender = NULL, receiver = NULL,
                            colors = NULL, colours = NULL, ...) {
  if (is.null(colors)) colors <- colours
  CCI_df <- x

  # Filter rows by sender (AND logic with receiver)
  if (!is.null(sender)) {
    CCI_df <- CCI_df[CCI_df$Sender %in% sender, , drop = FALSE]
    if (nrow(CCI_df) == 0)
      stop("No rows remaining after filtering by sender.")
  }

  # Filter rows by receiver (AND logic with sender)
  if (!is.null(receiver)) {
    CCI_df <- CCI_df[CCI_df$Receiver %in% receiver, , drop = FALSE]
    if (nrow(CCI_df) == 0)
      stop("No rows remaining after filtering by receiver.")
  }

  # Subset to top_lr LR pair columns (already ranked by sig_numbers)
  lr_cols <- setdiff(colnames(CCI_df), c("Sender", "Receiver"))
  if (!is.null(top_lr))
    lr_cols <- lr_cols[seq_len(min(top_lr, length(lr_cols)))]

  # Subset to top_pairs rows by max score across the displayed LR columns
  if (!is.null(top_pairs) && nrow(CCI_df) > top_pairs) {
    score_mat <- as.matrix(CCI_df[, lr_cols, drop = FALSE])
    row_max   <- apply(score_mat, 1, max, na.rm = TRUE)
    keep      <- order(row_max, decreasing = TRUE)[seq_len(top_pairs)]
    CCI_df    <- CCI_df[keep, , drop = FALSE]
  }

  senders   <- CCI_df$Sender
  receivers <- CCI_df$Receiver

  if (is.null(colors)) {
    all_cts <- sort(unique(c(senders, receivers)))
    colors  <- setNames(cols[seq_along(all_cts)], all_cts)
  }

  row_ha <- rowAnnotation(
    Sender = senders,
    Receiver = receivers,
    col = list(
      Sender = colors,
      Receiver = colors
    ),
    annotation_legend_param = list(Sender = list(show = FALSE),
                                   Receiver = list(show = FALSE)),
    annotation_name_gp = gpar(fontsize = 10, fontface = "bold"),
    annotation_name_side = "top"
  )

  row_labels <- paste(senders, receivers, sep = " -> ")

  f1 <- viridisLite::viridis(10)
  ht <- ComplexHeatmap::Heatmap(as.matrix(CCI_df[, lr_cols, drop = FALSE]),
                name = "Interaction\nScore",
                col = f1,
                cluster_rows = TRUE,
                cluster_columns = TRUE,
                row_labels = row_labels,
                row_names_gp = gpar(fontsize = 10, fontface = "bold"),
                column_names_gp = gpar(fontsize = 10, fontface = "bold"),
                column_names_rot = 45,
                heatmap_legend_param = list(title_position = "topcenter"),
                left_annotation = row_ha)

  draw(ht,
       heatmap_legend_side = "right",
       padding = unit(c(5, 15, 5, 10), "mm"))

  invisible(ht)
}

#' Create a blisa object
#'
#' Constructor for the \code{blisa} S3 class, which stores the full output of
#' \code{\link{blisa}}.
#'
#' @param LR_results Data frame of BLISA results, one row per ligand-receptor
#'   pair.
#' @param bins An \code{sf} object of hexagonal bin polygons.
#' @param spatial_weights Spatial weights list from
#'   \code{\link{computeSpatialWeights}}.
#' @param CCI_scores Wide data frame of cell-cell interaction scores from
#'   \code{\link{runCCI}}, or \code{NULL} if CCI was not computed.
#'
#' @return An object of class \code{blisa}.
#' @keywords internal
new_blisa <- function(LR_results, bins, spatial_weights, CCI_scores = NULL) {
  structure(
    list(
      LR_results      = LR_results,
      bins            = bins,
      spatial_weights = spatial_weights,
      CCI_scores      = CCI_scores
    ),
    class = "blisa"
  )
}


#' @export
print.blisa <- function(x, ...) {
  cat("A blisa object\n")
  cat(" LR pairs tested  :", nrow(x$LR_results), "\n")
  cat(" Significant pairs:", sum(x$LR_results$sig_numbers > 0), "\n")
  cat(" Bins             :", nrow(x$bins), "\n")
  cat(" CCI computed     :", !is.null(x$CCI_scores), "\n")
  invisible(x)
}


#' Test if an object is a blisa object
#'
#' @param x Any R object.
#' @return Logical.
#' @export
is.blisa <- function(x) inherits(x, "blisa")

#' Dot plot ranking LR pairs by number of significant hotspot bins
#'
#' Generic function for ranking LR pairs. Dispatches on the class of \code{x}:
#' \itemize{
#'   \item \code{plotLRrank.blisa} accepts a \code{blisa} object and uses its
#'     \code{LR_results} slot directly.
#'   \item \code{plotLRrank.data.frame} accepts the \code{LR_results} data
#'     frame directly.
#' }
#'
#' @param x A \code{blisa} object or a data frame of LR results. The data frame
#'   must contain columns \code{sig_numbers} and \code{annotation}.
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return A \code{ggplot} object.
#' @export
plotLRrank <- function(x, ...) UseMethod("plotLRrank")


#' @describeIn plotLRrank Method for a \code{blisa} object. Extracts
#'   \code{LR_results} and delegates to \code{plotLRrank.data.frame}.
#'
#' @param top Integer or \code{NULL}. Number of top LR pairs (by
#'   \code{sig_numbers}) to display. Default \code{30}.
#' @param pt_size Numeric. Point size passed to \code{geom_point}. Default 4.
#' @param flip Logical. When \code{TRUE}, LR pairs are placed on the x-axis
#'   and the hotspot count on the y-axis (vertical orientation). Default
#'   \code{FALSE} (LR pairs on y-axis, horizontal orientation).
#'
#' @export
plotLRrank.blisa <- function(x, top = 30, pt_size = 4, flip = FALSE, ...) {
  plotLRrank.data.frame(x$LR_results, top = top, pt_size = pt_size, flip = flip)
}


#' @describeIn plotLRrank Method for a data frame of LR results (e.g. the
#'   \code{LR_results} slot of a \code{blisa} object).
#'
#' @export
plotLRrank.data.frame <- function(x, top = 30, pt_size = 4, flip = FALSE, ...) {
  LR_results <- x

  if (!is.null(top)) {
    LR_results <- LR_results[seq_len(min(top, nrow(LR_results))), ]
  }

  n_shown <- nrow(LR_results)
  title   <- paste0("Top ", n_shown, " LR pairs by hotspot count")

  LR_results$LR_pair <- rownames(LR_results)
  LR_results <- LR_results[order(-LR_results$sig_numbers), ]
  # flip=FALSE: levels low->high so highest appears at top of y-axis
  # flip=TRUE:  levels high->low so highest appears at left of x-axis
  LR_results$LR_pair <- factor(LR_results$LR_pair,
                            levels = if (flip) LR_results$LR_pair else rev(LR_results$LR_pair))

  color_scale <- scale_color_manual(
    values = c(
      "Secreted Signaling"    = "#90a955",
      "ECM-Receptor"          = "#219ebc",
      "Cell-Cell Contact"     = "#f7b801",
      "Non-protein Signaling" = "#9f86c0"
    )
  )

  if (flip) {
    p <- ggplot(LR_results, aes(x = LR_pair, y = sig_numbers, color = annotation)) +
      geom_point(size = pt_size) +
      color_scale +
      scale_y_continuous(expand = expansion(add = 100)) +
      labs(x = "Ligand\u2013Receptor Pair", y = "Sig Spot Numbers",
           color = "Annotation", title = title) +
      theme_minimal() +
      theme(
        legend.position = "right",
        legend.title    = element_text(size = 12, face = "bold"),
        legend.text     = element_text(size = 10),
        panel.border    = element_rect(colour = "black", fill = NA, linewidth = 0.8),
        axis.text.x     = element_text(size = 12, angle = 45, hjust = 1),
        axis.text.y     = element_text(angle = 0, hjust = 1)
      ) +
      coord_cartesian(clip = "off")
  } else {
    p <- ggplot(LR_results, aes(x = sig_numbers, y = LR_pair, color = annotation)) +
      geom_point(size = pt_size) +
      color_scale +
      scale_x_continuous(expand = expansion(add = 100)) +
      labs(x = "Sig Spot Numbers", y = "Ligand\u2013Receptor Pair",
           color = "Annotation", title = title) +
      theme_minimal() +
      theme(
        legend.position = "right",
        legend.title    = element_text(size = 12, face = "bold"),
        legend.text     = element_text(size = 10),
        panel.border    = element_rect(colour = "black", fill = NA, linewidth = 0.8),
        axis.text.y     = element_text(size = 12),
        axis.text.x     = element_text(angle = 45, hjust = 1)
      ) +
      coord_cartesian(clip = "off")
  }

  return(p)
}

#' Sender-by-receiver heatmap of aggregated CCI scores across LR pairs
#'
#' Generic function. Aggregates CCI scores across all (or the top-ranked)
#' ligand-receptor pairs and draws a clustered receiver-by-sender heatmap, one
#' cell per Sender \eqn{\rightarrow} Receiver combination.
#'
#' @param x A \code{blisa} object or a CCI scores data frame (the
#'   \code{CCI_scores} slot of a \code{blisa} object).
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return A \code{Heatmap} object.
#' @seealso \code{\link{plotCCILR}} for a per-LR-pair version of this plot;
#'   \code{\link{plotCCI}} for a heatmap with LR pairs as columns.
#' @export
plotCCIsummary <- function(x, ...) UseMethod("plotCCIsummary")


#' @describeIn plotCCIsummary Method for a \code{blisa} object. Stops with an
#'   informative error if \code{CCI_scores} is \code{NULL}.
#'
#' @param top_lr Integer or \code{NULL}. Number of top-ranked LR pairs (by
#'   \code{sig_numbers}) to include before aggregating. LR pairs in
#'   \code{CCI_scores} are already ordered by rank, so this takes the first
#'   \code{top_lr} columns. \code{NULL} (default) uses all pairs.
#' @param sender Character vector or \code{NULL}. If provided, only rows where
#'   \code{Sender} is in this vector are kept (AND logic with \code{receiver}).
#'   Default \code{NULL} (all senders).
#' @param receiver Character vector or \code{NULL}. If provided, only rows
#'   where \code{Receiver} is in this vector are kept (AND logic with
#'   \code{sender}). Default \code{NULL} (all receivers).
#' @param agg_fun Function used to aggregate scores across LR pairs for each
#'   Sender \eqn{\rightarrow} Receiver combination. Receives a numeric vector
#'   with \code{NA}s already removed. Default \code{sum}.
#'
#' @export
plotCCIsummary.blisa <- function(x, top_lr = NULL, sender = NULL,
                                  receiver = NULL, agg_fun = sum, ...) {
  if (is.null(x$CCI_scores))
    stop("CCI_scores is NULL. Run runCCI() first to compute CCI scores.")
  plotCCIsummary.default(x$CCI_scores, top_lr = top_lr, sender = sender,
                          receiver = receiver, agg_fun = agg_fun)
}


#' @describeIn plotCCIsummary Method for a CCI scores data frame (e.g. the
#'   \code{CCI_scores} slot of a \code{blisa} object).
#'
#' @export
plotCCIsummary.default <- function(x, top_lr = NULL, sender = NULL,
                                    receiver = NULL, agg_fun = sum, ...) {
  CCI_df <- x

  if (!is.null(sender)) {
    CCI_df <- CCI_df[CCI_df$Sender %in% sender, , drop = FALSE]
    if (nrow(CCI_df) == 0)
      stop("No rows remaining after filtering by sender.")
  }

  if (!is.null(receiver)) {
    CCI_df <- CCI_df[CCI_df$Receiver %in% receiver, , drop = FALSE]
    if (nrow(CCI_df) == 0)
      stop("No rows remaining after filtering by receiver.")
  }

  lr_cols <- setdiff(colnames(CCI_df), c("Sender", "Receiver"))
  if (!is.null(top_lr))
    lr_cols <- lr_cols[seq_len(min(top_lr, length(lr_cols)))]

  score_mat <- as.matrix(CCI_df[, lr_cols, drop = FALSE])
  agg_scores <- apply(score_mat, 1, function(v) agg_fun(v[!is.na(v)]))

  senders   <- CCI_df$Sender
  receivers <- CCI_df$Receiver
  all_senders   <- unique(senders)
  all_receivers <- unique(receivers)

  interaction_mat <- matrix(NA,
                            nrow = length(all_receivers),
                            ncol = length(all_senders),
                            dimnames = list(all_receivers, all_senders))

  for (i in seq_along(agg_scores)) {
    interaction_mat[receivers[i], senders[i]] <- agg_scores[i]
  }

  f1 <- viridisLite::viridis(10)
  p <- Heatmap(interaction_mat,
               name = "Interaction\nScore",
               col = f1,
               cluster_rows = TRUE,
               cluster_columns = TRUE,
               row_title = "Receiver",
               column_title = "Sender",
               row_names_gp = gpar(fontsize = 12, fontface = "bold"),
               column_names_gp = gpar(fontsize = 12, fontface = "bold"),
               column_names_rot = 45,
               na_col = "grey90",
               heatmap_legend_param = list(title_position = "topcenter"))

  return(p)
}

#' Spatial map of dominant sender-receiver cell-type pairs at BLISA hotspots
#'
#' For a selected ligand-receptor pair, identifies the dominant interacting
#' cell-type pair at each significant hotspot bin and draws a spatial map of
#' the tissue coloured by those pairs. Receiver cells are those inside hotspot
#' bins; sender cells are drawn from the immediate neighbourhood.
#'
#' @param x A \code{blisa} object as returned by \code{\link{blisa}}.
#' @param counts_by_group Named list of gene-by-bin count matrices, one per
#'   cell type. Typically the \code{counts_by_group} element returned by
#'   \code{\link{hexBinCells}}. Names must match the cell-type levels.
#' @param index Integer. Row index into \code{x$LR_results} selecting the
#'   ligand-receptor pair to visualise. Ignored when both \code{ligand} and
#'   \code{receptor} are supplied. Default \code{1} (top-ranked pair).
#' @param ligand Character. Ligand gene symbol. When both \code{ligand} and
#'   \code{receptor} are provided the matching LR pair is located automatically
#'   and \code{index} is ignored. Must be supplied together with
#'   \code{receptor}.
#' @param receptor Character. Receptor gene symbol. Must be supplied together
#'   with \code{ligand}.
#' @param top_pairs Integer. Maximum number of distinct cell-type pairs to show
#'   in the legend; remaining pairs are grouped as \code{"rare pairs"}. Default
#'   \code{30}.
#'
#' @return A \code{ggplot} object.
#' @seealso \code{\link{plotHotspots}} for a significance-based spatial map of
#'   hotspot bins.
#' @export
plotCCIspatial <- function(x, counts_by_group, index = 1, ligand = NULL,
                            receptor = NULL, top_pairs = 30) {

  LR_results <- x$LR_results
  bins       <- x$bins
  sw         <- x$spatial_weights
  ct_names   <- names(counts_by_group)

  index   <- .resolve_lr_index(LR_results, index, ligand, receptor)
  lr_pair <- rownames(LR_results)[index]

  gene_l <- LR_results$ligand.symbol[index]
  gene_r <- LR_results$receptor.symbol[index]
  sigHH  <- LR_results$sig_index[[index]]
  mode   <- LR_results$ccc_mode[index]

  if (length(sigHH) == 0)
    stop("LR pair '", lr_pair, "' has no significant hotspots.")

  nb_list <- if (mode == "nearby") sw$queen_nb_full else sw$dist_nb_full

  # For each hotspot bin, score all sender-receiver cell-type combinations
  # and retain the dominant (highest-scoring) pair
  dominant_pairs <- do.call(rbind, lapply(sigHH, function(h) {
    nb_h <- unique(c(h, nb_list[[h]]))

    # Receptor expression in bin h per cell type (receivers)
    r_scores <- sapply(ct_names, function(ct)
      sum(counts_by_group[[ct]][gene_r, h]))

    # Ligand expression in neighbourhood of h per cell type (senders)
    l_scores <- sapply(ct_names, function(ct)
      sum(counts_by_group[[ct]][gene_l, nb_h]))

    score_mat <- 0.5 * log2(outer(r_scores, l_scores) + 1)
    best      <- which(score_mat == max(score_mat), arr.ind = TRUE)[1L, ]

    data.frame(
      hex_id    = h,
      cell_pair = paste(ct_names[best[2L]], ct_names[best[1L]], sep = " -> "),
      product   = score_mat[best[1L], best[2L]]
    )
  }))

  # Legend: show top N pairs by frequency; group remainder as "rare pairs"
  tbl <- sort(table(dominant_pairs$cell_pair), decreasing = TRUE)
  if (length(tbl) <= top_pairs) {
    shown_pairs  <- names(tbl)
    legend_title <- "All pairs"
  } else {
    shown_pairs  <- names(tbl[seq_len(top_pairs)])
    legend_title <- paste0("Top ", top_pairs, " pairs")
  }
  dominant_pairs$cell_pair_plot <- ifelse(dominant_pairs$cell_pair %in% shown_pairs,
                                     dominant_pairs$cell_pair, "rare pairs")

  # Label every bin: Empty / Non-Significant / dominant interacting pair
  bins$cell_pair_plot                   <- ifelse(bins$n_cells > 0,
                                                   "Non-Significant", "Empty")
  bins$cell_pair_plot[dominant_pairs$hex_id] <- dominant_pairs$cell_pair_plot
  legend_levels       <- c(shown_pairs, "rare pairs", "Non-Significant", "Empty")
  bins$cell_pair_plot <- factor(bins$cell_pair_plot, levels = legend_levels)

  fill_values <- c(
    setNames(cols[seq_along(shown_pairs)], shown_pairs),
    "rare pairs"      = "#404040",
    "Non-Significant" = "#D3D3D3",
    "Empty"           = "#F5F5F5"
  )

  p <- ggplot(bins) +
    geom_sf(aes(fill = cell_pair_plot), color = NA) +
    scale_fill_manual(values = fill_values, drop = FALSE) +
    guides(fill = guide_legend(title = legend_title)) +
    labs(title = paste0(lr_pair, ": Interacting Hotspots")) +
    theme_void()

  return(p)
}

#' Score cell-cell interactions from BLISA hotspots
#'
#' Generic function for scoring cell-cell interactions. Dispatches on the
#' class of \code{x}:
#' \itemize{
#'   \item \code{runCCI.blisa} accepts a \code{blisa} object. If
#'     \code{CCI_scores} are already present and \code{overwrite = FALSE}
#'     (the default), the object is returned unchanged. Set
#'     \code{overwrite = TRUE} with a \code{counts_by_group} to recompute and
#'     replace existing scores. If no scores exist, \code{counts_by_group}
#'     must be supplied and scores are computed and attached.
#'   \item \code{runCCI.default} performs the raw computation given a
#'     \code{blisa} object and a \code{counts_by_group} list, returning only
#'     the scores data frame. Used internally by \code{runCCI.blisa} and
#'     \code{\link{blisa.default}}.
#' }
#'
#' @param x A \code{blisa} object.
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return See individual method documentation.
#' @export
runCCI <- function(x, ...) UseMethod("runCCI")


#' @describeIn runCCI Method for a \code{blisa} object. If \code{CCI_scores}
#'   are already present and \code{overwrite = FALSE} (the default), the object
#'   is returned unchanged. Set \code{overwrite = TRUE} with a
#'   \code{counts_by_group} to recompute and replace existing scores. If no
#'   scores exist, \code{counts_by_group} must be supplied and scores are
#'   computed and attached to \code{x$CCI_scores}.
#'
#' @param counts_by_group Named list of gene-by-bin sparse count matrices, one
#'   per group level (e.g. cell type). Typically the \code{counts_by_group}
#'   element of the list returned by \code{\link{hexBinCells}} when
#'   \code{group} is supplied. Names must match the group levels. Required
#'   when \code{x$CCI_scores} is \code{NULL} or when \code{overwrite = TRUE}.
#' @param overwrite Logical. If \code{FALSE} (default) and \code{x$CCI_scores}
#'   is already populated, the object is returned unchanged. If \code{TRUE} and
#'   \code{counts_by_group} is supplied, existing scores are recomputed and
#'   replaced.
#'
#' @return \code{runCCI.blisa}: the input \code{blisa} object with
#'   \code{CCI_scores} populated (a wide data frame -- rows are
#'   \code{"Sender->Receiver"} group pairs, columns are LR pairs).
#'
#' @export
runCCI.blisa <- function(x, counts_by_group = NULL, overwrite = FALSE, ...) {
  if (!is.null(x$CCI_scores)) {
    if (!overwrite) {
      message("CCI_scores already present \u2014 returning object unchanged. ",
              "Set overwrite = TRUE to recompute.")
      return(x)
    }
    if (is.null(counts_by_group))
      stop(
        "counts_by_group is required to recompute CCI_scores. ",
        "Supply counts_by_group explicitly or set overwrite = FALSE to keep existing scores."
      )
    message("Recomputing CCI_scores (overwrite = TRUE)...")
    x$CCI_scores <- runCCI.default(x, counts_by_group)
    message("CCI_scores replaced.")
    return(x)
  }
  if (is.null(counts_by_group))
    stop(
      "counts_by_group is required when CCI_scores have not been computed. ",
      "Re-run blisa() with a 'group' argument, or supply counts_by_group explicitly."
    )
  message("Computing CCI_scores...")
  x$CCI_scores <- runCCI.default(x, counts_by_group)
  message("CCI_scores added.")
  x
}


#' @describeIn runCCI Default method. Performs the raw CCI computation and
#'   returns only the scores data frame. Typically called internally; use
#'   \code{runCCI.blisa} to compute and attach scores to a \code{blisa} object
#'   in one step.
#'
#' @return \code{runCCI.default}: a data frame with \code{"Sender->Receiver"}
#'   row names and one column per significant LR pair containing the
#'   interaction score \code{0.5 * log2(receiver * sender + 1)}.
#'
#' @export
runCCI.default <- function(x, counts_by_group, ...) {
  LRI_sum       <- x$LR_results
  sw            <- x$spatial_weights
  queen_nb_full <- sw$queen_nb_full
  dist_nb_full  <- sw$dist_nb_full
  ct_names      <- names(counts_by_group)

  scores_list <- lapply(seq_len(nrow(LRI_sum)), function(idx) {
    if (LRI_sum$sig_numbers[idx] == 0) return(NULL)

    l_gene <- LRI_sum$ligand.symbol[idx]
    r_gene <- LRI_sum$receptor.symbol[idx]
    mode   <- LRI_sum$ccc_mode[idx]
    sigHH  <- LRI_sum$sig_index[[idx]]

    nb_list  <- if (mode == "nearby") queen_nb_full else dist_nb_full
    sigHH_ng <- unique(c(sigHH, unlist(nb_list[sigHH])))

    # Sum receptor counts in hotspot bins per group (receivers)
    receiver_sums <- sapply(ct_names, function(ct) {
      sum(counts_by_group[[ct]][r_gene, sigHH])
    })

    # Sum ligand counts in hotspot + neighbour bins per group (senders)
    sender_sums <- sapply(ct_names, function(ct) {
      sum(counts_by_group[[ct]][l_gene, sigHH_ng])
    })

    score_mat  <- 0.5 * log2(outer(receiver_sums, sender_sums, FUN = "*") + 1)
    pair_names <- as.vector(outer(ct_names, ct_names, function(r, s) paste(s, r, sep = "->")))
    scores     <- as.vector(score_mat)
    names(scores) <- pair_names
    scores
  })
  names(scores_list) <- rownames(LRI_sum)

  scores_list <- Filter(Negate(is.null), scores_list)
  if (length(scores_list) == 0) return(data.frame())

  score_df <- as.data.frame(do.call(cbind, scores_list))
  # Split "Sender->Receiver" row names into explicit columns
  sr_pairs <- strsplit(rownames(score_df), "->")
  data.frame(
    Sender   = sapply(sr_pairs, `[`, 1),
    Receiver = sapply(sr_pairs, `[`, 2),
    score_df,
    row.names = NULL,
    check.names = FALSE
  )
}

#' Run BLISA spatial cell-cell communication analysis
#'
#' Generic function for running BLISA (Bivariate Local Indicator of Spatial
#' Association). Dispatches on the class of \code{x}:
#' \itemize{
#'   \item \code{blisa.default} accepts a pre-binned gene-by-bin count
#'     matrix and a matching \code{bins} polygon object.
#'   \item \code{blisa.SpatialExperiment} accepts a cell-level
#'     \code{SpatialExperiment} object and bins cells into hexagonal tiles
#'     internally via \code{\link{hexBinCells}} before running the analysis.
#' }
#'
#' @param x A gene-by-bin count matrix (for \code{blisa.default}) or a
#'   cell-level \code{SpatialExperiment} object (for
#'   \code{blisa.SpatialExperiment}).
#' @param ... Additional arguments passed to the relevant method.
#'
#' @return A list; see individual method documentation for details.
#' @export
blisa <- function(x, ...) UseMethod("blisa")


#' @describeIn blisa Method for a gene-by-bin count matrix.
#'
#' @param bins An \code{sf} object of bin polygons. Row order must match the
#'   column order of \code{x}.
#' @param LR_df Data frame of ligand-receptor pairs with columns
#'   \code{ligand.symbol} and \code{receptor.symbol}. When \code{NULL},
#'   CellChatDB for the chosen \code{species} is downloaded automatically.
#' @param bin_size Numeric. Bin spacing used to define queen adjacency for
#'   nearby-mode interactions. Default \code{50}.
#' @param dmax Numeric. Maximum distance for diffuse-mode neighbours.
#'   Default \code{250}.
#' @param nsim Integer. Number of permutations for Moran's I significance.
#'   Default \code{999}.
#' @param p_cutoff Numeric. P-value threshold for High-High hotspots.
#'   Default \code{0.05}.
#' @param min_ligand Numeric. Minimum ligand count threshold. Default \code{10}.
#' @param min_receptor Numeric. Minimum receptor count threshold.
#'   Default \code{10}.
#' @param min_cells Integer. Bins with fewer cells are excluded from
#'   Moran's I and assigned neutral statistics (\emph{p} = 1, LISA = 0).
#'   Ignored when \code{n_cells_col = NA}. Default \code{1}.
#' @param n_cells_col Character or \code{NA}. Column in \code{bins} holding
#'   per-bin cell counts used for \code{min_cells} filtering.
#'   Set to \code{NA} to skip (default).
#' @param annotation_col Character. Column in \code{LR_df} specifying
#'   interaction category used for communication-mode assignment. Default
#'   \code{"annotation"}.
#' @param default_mode Character. CCC mode assigned to LR pairs whose
#'   annotation does not match \code{diffuse_category}. Default
#'   \code{"diffuse"}.
#' @param diffuse_category Character vector of annotation categories treated
#'   as diffuse signalling.
#' @param species Character. Which CellChatDB to download when
#'   \code{LR_df = NULL}. One of \code{"human"} (default) or \code{"mouse"}.
#' @param genes Character vector of gene names to consider when matching
#'   ligand-receptor pairs. Only LR pairs whose ligand and receptor genes are
#'   all present in \code{genes} and pass the expression thresholds will be
#'   tested. Defaults to \code{rownames(x)} (all genes in the count matrix).
#'   Genes specified here but absent from \code{rownames(x)} are silently
#'   ignored.
#' @param counts_by_group Named list of gene-by-bin count matrices, one per
#'   group level (e.g. cell type), as returned by \code{\link{hexBinCells}}
#'   when \code{group} is supplied. When provided, \code{\link{runCCI}} is
#'   called automatically after the BLISA loop and its output is included in
#'   the result as \code{CCI_scores}. Default \code{NULL}.
#'
#' @return An object of class \code{blisa} with four components:
#' \describe{
#'   \item{LR_results}{Data frame of BLISA results for each LR pair, including
#'     \code{ccc_mode}, \code{sig_numbers}, \code{sig_index}, \code{sig_pval},
#'     \code{all_pval}, \code{all_lisa}, \code{all_quadrant}, and original
#'     columns from \code{LR_df}. \code{all_quadrant} is a character vector
#'     of \code{spdep::hotspot} quadrant labels (\code{"High-High"},
#'     \code{"Low-Low"}, etc.) for every bin; non-tested bins are \code{NA}.}
#'   \item{bins}{Bin-level \code{sf} object of hexagonal polygons.}
#'   \item{spatial_weights}{Spatial weights list from \code{\link{computeSpatialWeights}}.}
#'   \item{CCI_scores}{\code{NULL} unless \code{counts_by_group} is supplied,
#'     in which case a wide data frame of interaction scores from
#'     \code{\link{runCCI}}: rows are \code{"Sender->Receiver"} group pairs,
#'     columns are LR pairs.}
#' }
#' @export
blisa.default <- function(
    x,
    bins,
    LR_df             = NULL,
    bin_size         = 50,
    dmax             = 250,
    nsim             = 999,
    p_cutoff         = 0.05,
    min_ligand       = 10,
    min_receptor     = 10,
    min_cells        = 1,
    n_cells_col      = NA,
    annotation_col   = "annotation",
    default_mode      = "diffuse",
    diffuse_category  = c("Secreted Signaling", "Non-protein Signaling"),
    species           = c("human", "mouse"),
    genes             = NULL,
    counts_by_group   = NULL,
    ...
) {
  sw             <- computeSpatialWeights(bins, bin_size, dmax, min_cells, n_cells_col)
  queen_wt       <- sw$queen_wt
  dist_wt        <- sw$dist_wt
  keep_idx_queen <- sw$keep_idx_queen
  keep_idx_dist  <- sw$keep_idx_dist

  # Restrict gene panel for LR pair matching; default to all genes in x
  genes <- if (is.null(genes)) rownames(x) else intersect(genes, rownames(x))

  LR_df_filtered <- filterLRpairs(
    counts       = x[genes, , drop = FALSE],
    min_ligand   = min_ligand,
    min_receptor = min_receptor,
    LR_df        = LR_df,
    species      = species
  )

  LR_results <- LR_df_add_mode(LR_df_filtered, annotation_col, default_mode, diffuse_category)

  LR_results$sig_numbers   <- integer(nrow(LR_results))
  LR_results$sig_index     <- vector("list", nrow(LR_results))
  LR_results$sig_pval      <- vector("list", nrow(LR_results))
  LR_results$all_pval      <- vector("list", nrow(LR_results))
  LR_results$all_lisa      <- vector("list", nrow(LR_results))
  LR_results$all_quadrant  <- vector("list", nrow(LR_results))

  for (i in seq_len(nrow(LR_results))) {
    message(rownames(LR_results)[i])

    ligand   <- LR_results$ligand.symbol[i]
    receptor <- LR_results$receptor.symbol[i]
    mode     <- LR_results$ccc_mode[i]

    if (mode == "nearby") {
      wt       <- queen_wt
      keep_idx <- keep_idx_queen
    } else {
      wt       <- dist_wt
      keep_idx <- keep_idx_dist
    }
    message("ccc mode is ", mode)

    x_full <- get_min_expr(receptor, x)
    y_full <- get_min_expr(ligand, x)

    res_bv <- spdep::localmoran_bv(x_full[keep_idx], y_full[keep_idx], wt, nsim = nsim)

    full_pval <- rep(1, ncol(x))
    full_lisa <- rep(0, ncol(x))
    full_pval[keep_idx] <- res_bv[, "Pr(folded) Sim"]
    full_lisa[keep_idx] <- res_bv[, "Ibvi"]

    hs <- spdep::hotspot(
      res_bv,
      Prname        = "Pr(folded) Sim",
      cutoff        = p_cutoff,
      quadrant.type = "pysal",
      p.adjust      = "none"
    )

    hs_idx_hh <- !is.na(hs) & hs == "High-High"
    HH_idx    <- keep_idx[which(hs_idx_hh)]
    HH_pval   <- full_pval[HH_idx]

    full_quadrant <- rep(NA_character_, ncol(x))
    full_quadrant[keep_idx] <- as.character(hs)

    LR_results$sig_numbers[i]   <- length(HH_idx)
    LR_results$sig_index[[i]]   <- HH_idx
    LR_results$sig_pval[[i]]    <- HH_pval
    LR_results$all_pval[[i]]    <- full_pval
    LR_results$all_lisa[[i]]    <- full_lisa
    LR_results$all_quadrant[[i]] <- full_quadrant
    LR_results$ccc_mode[i]      <- mode
  }

  LR_results <- LR_results[order(-LR_results$sig_numbers), , drop = FALSE]
  front_cols <- c("ccc_mode", "sig_numbers", "sig_index", "sig_pval")
  LR_results <- LR_results[, c(front_cols, setdiff(colnames(LR_results), front_cols))]

  CCI_scores <- NULL
  if (!is.null(counts_by_group)) {
    message("Running CCI analysis...")
    tmp <- new_blisa(LR_results, bins, sw)
    CCI_scores <- runCCI.default(tmp, counts_by_group)
  }

  new_blisa(LR_results, bins, sw, CCI_scores)
}


#' @describeIn blisa Method for a cell-level SpatialExperiment object.
#'   Bins cells into hexagonal tiles via \code{\link{hexBinCells}} then
#'   delegates to \code{blisa.default}.
#'
#' @param bin_size Numeric. Width of each hexagonal bin in coordinate units
#'   (e.g. microns). Passed to \code{\link{hexBinCells}} and
#'   \code{\link{computeSpatialWeights}}. Default \code{50}.
#' @param group Character. Column name in \code{colData(x)} to use as the
#'   grouping variable (e.g. cell type) for per-group bin aggregation and
#'   downstream CCI analysis via \code{\link{runCCI}}. If the column is not
#'   found in \code{colData(x)}, a message is issued and CCI is skipped.
#'   Default \code{"cell_type"}.
#' @param genes Character vector of gene names to consider when matching
#'   ligand-receptor pairs. Defaults to \code{rownames(x)} (all genes in the
#'   \code{SpatialExperiment} object).
#'
#' @export
blisa.SpatialExperiment <- function(x, bin_size = 50, LR_df = NULL,
                                       group = "cell_type",
                                       genes = NULL, ...) {
  coords <- as.data.frame(SpatialExperiment::spatialCoords(x))

  # Resolve group vector from colData
  cd_cols <- colnames(SummarizedExperiment::colData(x))
  if (!is.null(group) && group %in% cd_cols) {
    group_vec <- SummarizedExperiment::colData(x)[[group]]
  } else {
    if (!is.null(group))
      message("Column '", group, "' not found in colData(x) \u2014 CCI analysis will be skipped.")
    group_vec <- NULL
  }

  if (is.null(genes)) genes <- rownames(x)

  binned <- hexBinCells(coords, SummarizedExperiment::assay(x, "counts"), bin_size = bin_size, group = group_vec)

  blisa.default(
    x               = binned$counts_matrix,
    bins          = binned$bins,
    LR_df           = LR_df,
    bin_size        = bin_size,
    n_cells_col     = "n_cells",
    genes           = genes,
    counts_by_group = binned$counts_by_group,
    ...
  )
}

#' Bin cells into hexagonal spatial bins
#'
#' Aggregates single-cell spatial data into hexagonal bins and returns a
#' bin-level count matrix together with a matching \code{sf} polygon object,
#' ready to pass directly to \code{\link{blisa.default}}.
#'
#' @param coords_df Data frame or matrix with columns \code{x_centroid} and
#'   \code{y_centroid} (e.g. the output of \code{SpatialExperiment::spatialCoords()}).
#'   Row names must be cell IDs matching the column names of \code{counts_matrix}.
#' @param counts_matrix Gene-by-cell count matrix (dense or sparse). Row names
#'   must be gene symbols; column names must be cell IDs present in
#'   \code{coords_df}.
#' @param bin_size Numeric. Approximate width of each hexagonal bin in
#'   coordinate units (e.g. microns). Analogous to \code{grid.length.x} in
#'   \code{sciderHex::gridDensity}. Default \code{50}.
#' @param min_cells Integer. Bins containing fewer than \code{min_cells} cells
#'   are dropped from the output. Default \code{1}.
#' @param group Factor or character vector of length \code{ncol(counts_matrix)}
#'   giving the cell-type label of each cell. When supplied, a named list of
#'   per-cell-type gene-by-bin matrices is included in the output as
#'   \code{counts_by_group}. Default \code{NULL} (not computed).
#'
#' @return A list with:
#' \describe{
#'   \item{counts_matrix}{Gene-by-bin sparse count matrix (all cells combined).
#'     Column \emph{i} corresponds to row \emph{i} of \code{bins}.}
#'   \item{bins}{An \code{sf} object of hexagonal bin polygons with an
#'     \code{n_cells} column recording how many cells fall in each bin. Row
#'     order matches the columns of \code{counts_matrix}.}
#'   \item{counts_by_group}{(Only when \code{group} is supplied.) A named list
#'     of gene-by-bin sparse matrices, one per cell-type level, with the same
#'     bin order as \code{counts_matrix}.}
#' }
#' @export
hexBinCells <- function(coords_df, counts_matrix, bin_size = 50, min_cells = 1,
                        group = NULL) {

  if (!all(c("x_centroid", "y_centroid") %in% colnames(coords_df)))
    stop("coords_df must contain columns 'x_centroid' and 'y_centroid'.")
  if (!all(colnames(counts_matrix) %in% rownames(coords_df)))
    stop("All column names of counts_matrix must appear as row names in coords_df.")
  if (!is.null(group)) {
    if (length(group) != ncol(counts_matrix))
      stop("'group' must have the same length as ncol(counts_matrix).")
    group <- as.factor(group)
  }

  # Build sf point layer in the same order as counts_matrix columns
  coords_df <- as.data.frame(coords_df)
  cell_sf <- sf::st_as_sf(
    coords_df[colnames(counts_matrix), , drop = FALSE],
    coords = c("x_centroid", "y_centroid"),
    crs    = NA
  )

  # Hexagonal grid covering the full extent of the cells
  hex_geom <- sf::st_make_grid(
    cell_sf,
    cellsize = bin_size,
    what     = "polygons",
    square   = FALSE
  )
  bins <- sf::st_sf(bin_id = seq_along(hex_geom), geometry = hex_geom)

  # Point-in-polygon: assign each cell to its bin (same row order as cell_sf)
  cell_bins <- sf::st_join(cell_sf, bins, join = sf::st_intersects)
  cell_to_bin <- sf::st_drop_geometry(cell_bins)$bin_id

  # Aggregate counts: genes x bins  (H is cells x bins)
  n_bins     <- nrow(bins)
  bin_factor <- factor(cell_to_bin, levels = seq_len(n_bins))
  H          <- Matrix::sparse.model.matrix(~ bin_factor - 1)
  bin_counts <- counts_matrix %*% H

  bins$n_cells <- as.numeric(Matrix::colSums(H))

  # Drop bins below min_cells
  keep       <- bins$n_cells >= min_cells
  bins     <- bins[keep, , drop = FALSE]
  bin_counts <- bin_counts[, keep, drop = FALSE]

  result <- list(counts_matrix = bin_counts, bins = bins)

  # Per-cell-type bin matrices (reuse H, subset rows by cell type)
  if (!is.null(group)) {
    result$counts_by_group <- lapply(levels(group), function(ct) {
      idx <- which(group == ct)
      counts_matrix[, idx, drop = FALSE] %*% H[idx, keep, drop = FALSE]
    })
    names(result$counts_by_group) <- levels(group)
  }

  result
}

---
title: "BLISA Workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BLISA Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4,
  eval = FALSE
)
```

# Introduction

BLISA identifies spatially enriched ligand–receptor interactions from spatial transcriptomics data using local bivariate Moran’s I statistics.

This vignette demonstrates a typical BLISA workflow using Xenium data.

# Load Package

```{r}
library(blisa)
library(scider)
library(CellChat)
library(patchwork)
```

# Load Example Data

Example Xenium dataset distributed with the package:

```{r}
spe <- readRDS(
  system.file("extdata", "xenium380Cell_spe.rds", package = "blisa")
)

dim(spe)
```

# Visualize Spatial Cell Types

```{r}
cell_type_colors <- c(
  RColorBrewer::brewer.pal(7, "Set1"),
  RColorBrewer::brewer.pal(6, "Set2")
)

names(cell_type_colors) <- unique(spe$cell_type)

scider::plotSpatial(
  spe,
  group.by = "cell_type",
  pt.size = 0.2,
  pt.alpha = 0.9,
  cols = cell_type_colors
)
```

# Filter Ligand–Receptor Pairs

Restrict CellChat ligand–receptor database to genes present in the panel:

```{r}
counts_matrix <- SummarizedExperiment::assay(spe, "counts")

LR_pairs_filtered <- filterLRpairs(
  counts = counts_matrix,
  min_ligand = 1,
  min_receptor = 1,
  LR_df = CellChatDB.human$interaction
)

head(LR_pairs_filtered)
```

# Run BLISA

```{r}
system.time(BLISAres<- runBLISA.spe.isolates.removed(
    spe, 
    LR_df = LR_pairs_filtered))

BLISAres$LR_out
```

# Summarize Ligand–Receptor Interaction Enrichment

```{r}
plotLRIsum(BLISAres$LR_out)
```

# Visualize Spatial Distribution of Top LR Pairs

```{r, fig.width=15, fig.height=12}
plots <- lapply(seq_len(nrow(BLISAres$LR_results)), function(i) {
  plotHotspots(BLISAres, index = i)
})

combined_plot <- patchwork::wrap_plots(plots, ncol = 3)

combined_plot

pdf("xenium_LRI_spatial_plots.pdf", width = 15, height = 12)
print(combined_plot)
dev.off()
```

# Run Cell–Cell Interaction Analysis

When `blisa()` is called with a `group` argument (e.g. `group = "cell_type"`),
CCI scores are computed automatically and stored in `BLISAres$CCI_scores`.
To compute or recompute them after the fact, call `runCCI()` on the
`blisa` object and supply `counts_by_group` from `hexBinCells()`:

```{r}
# CCI already computed inside blisa() — retrieve directly
CCIres <- BLISAres$CCI_scores

# Alternatively, compute on an existing blisa object that lacks CCI_scores:
# binned  <- hexBinCells(coords, counts_matrix, bin_size = 50, group = cell_type_vec)
# BLISAres <- runCCI(BLISAres, counts_by_group = binned$counts_by_group)
# CCIres  <- BLISAres$CCI_scores

head(CCIres)
```

# Visualize CCI Heatmap

```{r fig.height=8, fig.width=8}
plotCCI(BLISAres)
```

```{r, fig.height=10, fig.width=9}
plotCCI(BLISAres, sender = "Invasive_Tumor", receiver = "Invasive_Tumor")
```

# Visualize One Ligand–Receptor Pair

```{r fig.height=5, fig.width=6}
p <- plotCCIpair(BLISAres, ligand = "CXCL12", receptor = "CXCR4")

draw(
  p,
  heatmap_legend_side = "right",
  padding = unit(c(5, 5, 10, 10), "mm")
)
```

```{r fig.height=7, fig.width=10}
CCIspatial(
  spe,
  BLISAres,
  index = 2
)
```

# Session Information

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runCCI.R
\name{runCCI}
\alias{runCCI}
\alias{runCCI.blisa}
\alias{runCCI.default}
\title{Score cell-cell interactions from BLISA hotspots}
\usage{
runCCI(x, ...)

\method{runCCI}{blisa}(x, counts_by_group = NULL, overwrite = FALSE, ...)

\method{runCCI}{default}(x, counts_by_group, ...)
}
\arguments{
\item{x}{A \code{blisa} object.}

\item{...}{Additional arguments passed to the relevant method.}

\item{counts_by_group}{Named list of gene-by-bin sparse count matrices, one
per group level (e.g. cell type). Typically the \code{counts_by_group}
element of the list returned by \code{\link{hexBinCells}} when
\code{group} is supplied. Names must match the group levels. Required
when \code{x$CCI_scores} is \code{NULL} or when \code{overwrite = TRUE}.}

\item{overwrite}{Logical. If \code{FALSE} (default) and \code{x$CCI_scores}
is already populated, the object is returned unchanged. If \code{TRUE} and
\code{counts_by_group} is supplied, existing scores are recomputed and
replaced.}
}
\value{
See individual method documentation.

\code{runCCI.blisa}: the input \code{blisa} object with
\code{CCI_scores} populated (a wide data frame -- rows are
\code{"Sender->Receiver"} group pairs, columns are LR pairs).

\code{runCCI.default}: a data frame with \code{"Sender->Receiver"}
row names and one column per significant LR pair containing the
interaction score \code{0.5 * log2(receiver * sender + 1)}.
}
\description{
Generic function for scoring cell-cell interactions. Dispatches on the
class of \code{x}:
\itemize{
\item \code{runCCI.blisa} accepts a \code{blisa} object. If
\code{CCI_scores} are already present and \code{overwrite = FALSE}
(the default), the object is returned unchanged. Set
\code{overwrite = TRUE} with a \code{counts_by_group} to recompute and
replace existing scores. If no scores exist, \code{counts_by_group}
must be supplied and scores are computed and attached.
\item \code{runCCI.default} performs the raw computation given a
\code{blisa} object and a \code{counts_by_group} list, returning only
the scores data frame. Used internally by \code{runCCI.blisa} and
\code{\link{blisa.default}}.
}
}
\section{Methods (by class)}{
\itemize{
\item \code{runCCI(blisa)}: Method for a \code{blisa} object. If \code{CCI_scores}
are already present and \code{overwrite = FALSE} (the default), the object
is returned unchanged. Set \code{overwrite = TRUE} with a
\code{counts_by_group} to recompute and replace existing scores. If no
scores exist, \code{counts_by_group} must be supplied and scores are
computed and attached to \code{x$CCI_scores}.

\item \code{runCCI(default)}: Default method. Performs the raw CCI computation and
returns only the scores data frame. Typically called internally; use
\code{runCCI.blisa} to compute and attach scores to a \code{blisa} object
in one step.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotLRrank.R
\name{plotLRrank}
\alias{plotLRrank}
\alias{plotLRrank.blisa}
\alias{plotLRrank.data.frame}
\title{Dot plot ranking LR pairs by number of significant hotspot bins}
\usage{
plotLRrank(x, ...)

\method{plotLRrank}{blisa}(x, top = 30, pt_size = 4, flip = FALSE, ...)

\method{plotLRrank}{data.frame}(x, top = 30, pt_size = 4, flip = FALSE, ...)
}
\arguments{
\item{x}{A \code{blisa} object or a data frame of LR results. The data frame
must contain columns \code{sig_numbers} and \code{annotation}.}

\item{...}{Additional arguments passed to the relevant method.}

\item{top}{Integer or \code{NULL}. Number of top LR pairs (by
\code{sig_numbers}) to display. Default \code{30}.}

\item{pt_size}{Numeric. Point size passed to \code{geom_point}. Default 4.}

\item{flip}{Logical. When \code{TRUE}, LR pairs are placed on the x-axis
and the hotspot count on the y-axis (vertical orientation). Default
\code{FALSE} (LR pairs on y-axis, horizontal orientation).}
}
\value{
A \code{ggplot} object.
}
\description{
Generic function for ranking LR pairs. Dispatches on the class of \code{x}:
\itemize{
\item \code{plotLRrank.blisa} accepts a \code{blisa} object and uses its
\code{LR_results} slot directly.
\item \code{plotLRrank.data.frame} accepts the \code{LR_results} data
frame directly.
}
}
\section{Methods (by class)}{
\itemize{
\item \code{plotLRrank(blisa)}: Method for a \code{blisa} object. Extracts
\code{LR_results} and delegates to \code{plotLRrank.data.frame}.

\item \code{plotLRrank(data.frame)}: Method for a data frame of LR results (e.g. the
\code{LR_results} slot of a \code{blisa} object).

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blisa-class.R
\name{is.blisa}
\alias{is.blisa}
\title{Test if an object is a blisa object}
\usage{
is.blisa(x)
}
\arguments{
\item{x}{Any R object.}
}
\value{
Logical.
}
\description{
Test if an object is a blisa object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCCIspatial.R
\name{plotCCIspatial}
\alias{plotCCIspatial}
\title{Spatial map of dominant sender-receiver cell-type pairs at BLISA hotspots}
\usage{
plotCCIspatial(
  x,
  counts_by_group,
  index = 1,
  ligand = NULL,
  receptor = NULL,
  top_pairs = 30
)
}
\arguments{
\item{x}{A \code{blisa} object as returned by \code{\link{blisa}}.}

\item{counts_by_group}{Named list of gene-by-bin count matrices, one per
cell type. Typically the \code{counts_by_group} element returned by
\code{\link{hexBinCells}}. Names must match the cell-type levels.}

\item{index}{Integer. Row index into \code{x$LR_results} selecting the
ligand-receptor pair to visualise. Ignored when both \code{ligand} and
\code{receptor} are supplied. Default \code{1} (top-ranked pair).}

\item{ligand}{Character. Ligand gene symbol. When both \code{ligand} and
\code{receptor} are provided the matching LR pair is located automatically
and \code{index} is ignored. Must be supplied together with
\code{receptor}.}

\item{receptor}{Character. Receptor gene symbol. Must be supplied together
with \code{ligand}.}

\item{top_pairs}{Integer. Maximum number of distinct cell-type pairs to show
in the legend; remaining pairs are grouped as \code{"rare pairs"}. Default
\code{30}.}
}
\value{
A \code{ggplot} object.
}
\description{
For a selected ligand-receptor pair, identifies the dominant interacting
cell-type pair at each significant hotspot bin and draws a spatial map of
the tissue coloured by those pairs. Receiver cells are those inside hotspot
bins; sender cells are drawn from the immediate neighbourhood.
}
\seealso{
\code{\link{plotHotspots}} for a significance-based spatial map of
hotspot bins.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSpatialWeights.R
\name{computeSpatialWeights}
\alias{computeSpatialWeights}
\title{Compute Spatial Weights for BLISA}
\usage{
computeSpatialWeights(
  bins,
  bin_size = 50,
  dmax = 250,
  min_cells = 1,
  n_cells_col = NA
)
}
\arguments{
\item{bins}{An \code{sf} object of spatial bins.}

\item{bin_size}{Numeric. Bin spacing used to define queen adjacency
(\code{1.2 * bin_size} radius).}

\item{dmax}{Numeric. Maximum distance for diffuse-mode neighbours.}

\item{min_cells}{Integer. Minimum cell count for a bin to be
included. Ignored when \code{n_cells_col = NA}.}

\item{n_cells_col}{Character or \code{NA}. Column name in \code{bins}
holding per-bin cell counts. Set to \code{NA} to skip cell-count
filtering (default).}
}
\value{
A list with:
\describe{
\item{queen_wt}{Spatial weights list for nearby (queen) mode.}
\item{dist_wt}{Spatial weights list for diffuse (distance-decay) mode.}
\item{keep_idx_queen}{Integer indices of bins used in queen-mode Moran.}
\item{keep_idx_dist}{Integer indices of bins used in diffuse-mode Moran.}
\item{isolate_idx_queen}{Integer indices of original queen-mode isolates.}
\item{isolate_idx_dist}{Integer indices of original diffuse-mode isolates.}
\item{low_cell_idx}{Integer indices of bins excluded for low cell counts.}
\item{queen_nb_full}{Full (unsubset) neighbour list for nearby mode, indexed over all bins.}
\item{dist_nb_full}{Full (unsubset) neighbour list for diffuse mode, indexed over all bins.}
}
}
\description{
Builds queen (nearby) and distance-decay (diffuse) spatial weight matrices
from a bin-level \code{sf} object, excluding isolated bins and optionally
excluding low-cell bins. A second-pass isolation check further removes bins
that become isolated after the initial subset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCCI.R
\name{plotCCI}
\alias{plotCCI}
\alias{plotCCI.blisa}
\alias{plotCCI.default}
\title{Heatmap of CCI scores across all ligand-receptor pairs}
\usage{
plotCCI(x, ...)

\method{plotCCI}{blisa}(
  x,
  top_lr = 20,
  top_pairs = 30,
  sender = NULL,
  receiver = NULL,
  colors = NULL,
  colours = NULL,
  ...
)

\method{plotCCI}{default}(
  x,
  top_lr = 20,
  top_pairs = 30,
  sender = NULL,
  receiver = NULL,
  colors = NULL,
  colours = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{blisa} object or a CCI scores data frame (the
\code{CCI_scores} slot of a \code{blisa} object). The data frame must
contain columns \code{Sender}, \code{Receiver}, and one column per LR pair.}

\item{...}{Additional arguments passed to the relevant method.}

\item{top_lr}{Integer or \code{NULL}. Number of top-ranked LR pairs (by
\code{sig_numbers}) to display as columns. LR pairs in \code{CCI_scores}
are already ordered by rank, so this simply takes the first \code{top_lr}
columns. Default \code{20}.}

\item{top_pairs}{Integer or \code{NULL}. Number of top sender-receiver pairs
to display as rows, ranked by their maximum interaction score across the
displayed LR pairs (after \code{top_lr} is applied). When \code{NULL} all
rows are shown. Default \code{30}.}

\item{sender}{Character vector or \code{NULL}. If provided, only rows where
\code{Sender} is in this vector are kept. Applied independently of
\code{receiver} (AND logic when both are supplied). Default \code{NULL}
(all senders).}

\item{receiver}{Character vector or \code{NULL}. If provided, only rows
where \code{Receiver} is in this vector are kept. Applied independently of
\code{sender} (AND logic when both are supplied). Default \code{NULL}
(all receivers).}

\item{colors}{Named character vector mapping cell-type names to colours,
used for the sender/receiver row annotations. When \code{NULL} (default),
colours are assigned automatically from the package palette. The British
spelling \code{colours} is accepted as an alias.}

\item{colours}{Alias for \code{colors} (British spelling). Ignored when
\code{colors} is supplied.}
}
\value{
Invisibly returns the \code{Heatmap} object.
}
\description{
Generic function. Draws a clustered heatmap (via \code{ComplexHeatmap})
with rows as Sender \eqn{\rightarrow} Receiver cell-type pairs and columns
as LR pairs. Row annotations colour-code the sender and receiver cell types.
}
\section{Methods (by class)}{
\itemize{
\item \code{plotCCI(blisa)}: Method for a \code{blisa} object. Extracts
\code{CCI_scores} and delegates to \code{plotCCI.default}. Stops with an
informative error if \code{CCI_scores} is \code{NULL}.

\item \code{plotCCI(default)}: Method for a CCI scores data frame (e.g. the
\code{CCI_scores} slot of a \code{blisa} object).

}}
\seealso{
\code{\link{plotCCILR}} for a sender-by-receiver heatmap of a
single LR pair; \code{\link{plotCCIsummary}} for an aggregated
sender-by-receiver heatmap across all LR pairs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imports.R
\docType{package}
\name{blisa-package}
\alias{blisa-package}
\title{blisa: Bivariate Local Indicator of Spatial Association for spatial transcriptomics}
\description{
Tools for spatial cell-cell communication analysis of spatial transcriptomics data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCCIsummary.R
\name{plotCCIsummary}
\alias{plotCCIsummary}
\alias{plotCCIsummary.blisa}
\alias{plotCCIsummary.default}
\title{Sender-by-receiver heatmap of aggregated CCI scores across LR pairs}
\usage{
plotCCIsummary(x, ...)

\method{plotCCIsummary}{blisa}(
  x,
  top_lr = NULL,
  sender = NULL,
  receiver = NULL,
  agg_fun = sum,
  ...
)

\method{plotCCIsummary}{default}(
  x,
  top_lr = NULL,
  sender = NULL,
  receiver = NULL,
  agg_fun = sum,
  ...
)
}
\arguments{
\item{x}{A \code{blisa} object or a CCI scores data frame (the
\code{CCI_scores} slot of a \code{blisa} object).}

\item{...}{Additional arguments passed to the relevant method.}

\item{top_lr}{Integer or \code{NULL}. Number of top-ranked LR pairs (by
\code{sig_numbers}) to include before aggregating. LR pairs in
\code{CCI_scores} are already ordered by rank, so this takes the first
\code{top_lr} columns. \code{NULL} (default) uses all pairs.}

\item{sender}{Character vector or \code{NULL}. If provided, only rows where
\code{Sender} is in this vector are kept (AND logic with \code{receiver}).
Default \code{NULL} (all senders).}

\item{receiver}{Character vector or \code{NULL}. If provided, only rows
where \code{Receiver} is in this vector are kept (AND logic with
\code{sender}). Default \code{NULL} (all receivers).}

\item{agg_fun}{Function used to aggregate scores across LR pairs for each
Sender \eqn{\rightarrow} Receiver combination. Receives a numeric vector
with \code{NA}s already removed. Default \code{sum}.}
}
\value{
A \code{Heatmap} object.
}
\description{
Generic function. Aggregates CCI scores across all (or the top-ranked)
ligand-receptor pairs and draws a clustered receiver-by-sender heatmap, one
cell per Sender \eqn{\rightarrow} Receiver combination.
}
\section{Methods (by class)}{
\itemize{
\item \code{plotCCIsummary(blisa)}: Method for a \code{blisa} object. Stops with an
informative error if \code{CCI_scores} is \code{NULL}.

\item \code{plotCCIsummary(default)}: Method for a CCI scores data frame (e.g. the
\code{CCI_scores} slot of a \code{blisa} object).

}}
\seealso{
\code{\link{plotCCILR}} for a per-LR-pair version of this plot;
\code{\link{plotCCI}} for a heatmap with LR pairs as columns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blisa.R
\name{blisa}
\alias{blisa}
\alias{blisa.default}
\alias{blisa.SpatialExperiment}
\title{Run BLISA spatial cell-cell communication analysis}
\usage{
blisa(x, ...)

\method{blisa}{default}(
  x,
  bins,
  LR_df = NULL,
  bin_size = 50,
  dmax = 250,
  nsim = 999,
  p_cutoff = 0.05,
  min_ligand = 10,
  min_receptor = 10,
  min_cells = 1,
  n_cells_col = NA,
  annotation_col = "annotation",
  default_mode = "diffuse",
  diffuse_category = c("Secreted Signaling", "Non-protein Signaling"),
  species = c("human", "mouse"),
  genes = NULL,
  counts_by_group = NULL,
  ...
)

\method{blisa}{SpatialExperiment}(x, bin_size = 50, LR_df = NULL, group = "cell_type", genes = NULL, ...)
}
\arguments{
\item{x}{A gene-by-bin count matrix (for \code{blisa.default}) or a
cell-level \code{SpatialExperiment} object (for
\code{blisa.SpatialExperiment}).}

\item{...}{Additional arguments passed to the relevant method.}

\item{bins}{An \code{sf} object of bin polygons. Row order must match the
column order of \code{x}.}

\item{LR_df}{Data frame of ligand-receptor pairs with columns
\code{ligand.symbol} and \code{receptor.symbol}. When \code{NULL},
CellChatDB for the chosen \code{species} is downloaded automatically.}

\item{bin_size}{Numeric. Width of each hexagonal bin in coordinate units
(e.g. microns). Passed to \code{\link{hexBinCells}} and
\code{\link{computeSpatialWeights}}. Default \code{50}.}

\item{dmax}{Numeric. Maximum distance for diffuse-mode neighbours.
Default \code{250}.}

\item{nsim}{Integer. Number of permutations for Moran's I significance.
Default \code{999}.}

\item{p_cutoff}{Numeric. P-value threshold for High-High hotspots.
Default \code{0.05}.}

\item{min_ligand}{Numeric. Minimum ligand count threshold. Default \code{10}.}

\item{min_receptor}{Numeric. Minimum receptor count threshold.
Default \code{10}.}

\item{min_cells}{Integer. Bins with fewer cells are excluded from
Moran's I and assigned neutral statistics (\emph{p} = 1, LISA = 0).
Ignored when \code{n_cells_col = NA}. Default \code{1}.}

\item{n_cells_col}{Character or \code{NA}. Column in \code{bins} holding
per-bin cell counts used for \code{min_cells} filtering.
Set to \code{NA} to skip (default).}

\item{annotation_col}{Character. Column in \code{LR_df} specifying
interaction category used for communication-mode assignment. Default
\code{"annotation"}.}

\item{default_mode}{Character. CCC mode assigned to LR pairs whose
annotation does not match \code{diffuse_category}. Default
\code{"diffuse"}.}

\item{diffuse_category}{Character vector of annotation categories treated
as diffuse signalling.}

\item{species}{Character. Which CellChatDB to download when
\code{LR_df = NULL}. One of \code{"human"} (default) or \code{"mouse"}.}

\item{genes}{Character vector of gene names to consider when matching
ligand-receptor pairs. Defaults to \code{rownames(x)} (all genes in the
\code{SpatialExperiment} object).}

\item{counts_by_group}{Named list of gene-by-bin count matrices, one per
group level (e.g. cell type), as returned by \code{\link{hexBinCells}}
when \code{group} is supplied. When provided, \code{\link{runCCI}} is
called automatically after the BLISA loop and its output is included in
the result as \code{CCI_scores}. Default \code{NULL}.}

\item{group}{Character. Column name in \code{colData(x)} to use as the
grouping variable (e.g. cell type) for per-group bin aggregation and
downstream CCI analysis via \code{\link{runCCI}}. If the column is not
found in \code{colData(x)}, a message is issued and CCI is skipped.
Default \code{"cell_type"}.}
}
\value{
A list; see individual method documentation for details.

An object of class \code{blisa} with four components:
\describe{
\item{LR_results}{Data frame of BLISA results for each LR pair, including
\code{ccc_mode}, \code{sig_numbers}, \code{sig_index}, \code{sig_pval},
\code{all_pval}, \code{all_lisa}, \code{all_quadrant}, and original
columns from \code{LR_df}. \code{all_quadrant} is a character vector
of \code{spdep::hotspot} quadrant labels (\code{"High-High"},
\code{"Low-Low"}, etc.) for every bin; non-tested bins are \code{NA}.}
\item{bins}{Bin-level \code{sf} object of hexagonal polygons.}
\item{spatial_weights}{Spatial weights list from \code{\link{computeSpatialWeights}}.}
\item{CCI_scores}{\code{NULL} unless \code{counts_by_group} is supplied,
in which case a wide data frame of interaction scores from
\code{\link{runCCI}}: rows are \code{"Sender->Receiver"} group pairs,
columns are LR pairs.}
}
}
\description{
Generic function for running BLISA (Bivariate Local Indicator of Spatial
Association). Dispatches on the class of \code{x}:
\itemize{
\item \code{blisa.default} accepts a pre-binned gene-by-bin count
matrix and a matching \code{bins} polygon object.
\item \code{blisa.SpatialExperiment} accepts a cell-level
\code{SpatialExperiment} object and bins cells into hexagonal tiles
internally via \code{\link{hexBinCells}} before running the analysis.
}
}
\section{Methods (by class)}{
\itemize{
\item \code{blisa(default)}: Method for a gene-by-bin count matrix.

\item \code{blisa(SpatialExperiment)}: Method for a cell-level SpatialExperiment object.
Bins cells into hexagonal tiles via \code{\link{hexBinCells}} then
delegates to \code{blisa.default}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hexBinCells.R
\name{hexBinCells}
\alias{hexBinCells}
\title{Bin cells into hexagonal spatial bins}
\usage{
hexBinCells(
  coords_df,
  counts_matrix,
  bin_size = 50,
  min_cells = 1,
  group = NULL
)
}
\arguments{
\item{coords_df}{Data frame or matrix with columns \code{x_centroid} and
\code{y_centroid} (e.g. the output of \code{SpatialExperiment::spatialCoords()}).
Row names must be cell IDs matching the column names of \code{counts_matrix}.}

\item{counts_matrix}{Gene-by-cell count matrix (dense or sparse). Row names
must be gene symbols; column names must be cell IDs present in
\code{coords_df}.}

\item{bin_size}{Numeric. Approximate width of each hexagonal bin in
coordinate units (e.g. microns). Analogous to \code{grid.length.x} in
\code{sciderHex::gridDensity}. Default \code{50}.}

\item{min_cells}{Integer. Bins containing fewer than \code{min_cells} cells
are dropped from the output. Default \code{1}.}

\item{group}{Factor or character vector of length \code{ncol(counts_matrix)}
giving the cell-type label of each cell. When supplied, a named list of
per-cell-type gene-by-bin matrices is included in the output as
\code{counts_by_group}. Default \code{NULL} (not computed).}
}
\value{
A list with:
\describe{
\item{counts_matrix}{Gene-by-bin sparse count matrix (all cells combined).
Column \emph{i} corresponds to row \emph{i} of \code{bins}.}
\item{bins}{An \code{sf} object of hexagonal bin polygons with an
\code{n_cells} column recording how many cells fall in each bin. Row
order matches the columns of \code{counts_matrix}.}
\item{counts_by_group}{(Only when \code{group} is supplied.) A named list
of gene-by-bin sparse matrices, one per cell-type level, with the same
bin order as \code{counts_matrix}.}
}
}
\description{
Aggregates single-cell spatial data into hexagonal bins and returns a
bin-level count matrix together with a matching \code{sf} polygon object,
ready to pass directly to \code{\link{blisa.default}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCCILR.R
\name{plotCCILR}
\alias{plotCCILR}
\alias{plotCCILR.blisa}
\alias{plotCCILR.default}
\title{Sender-by-receiver heatmap of CCI scores for one ligand-receptor pair}
\usage{
plotCCILR(x, ...)

\method{plotCCILR}{blisa}(x, index = 1, ligand = NULL, receptor = NULL, ...)

\method{plotCCILR}{default}(x, lr_pair, ...)
}
\arguments{
\item{x}{A \code{blisa} object or a CCI scores data frame (the
\code{CCI_scores} slot of a \code{blisa} object).}

\item{...}{Additional arguments passed to the relevant method.}

\item{index}{Integer. Row index into \code{LR_results} selecting the
ligand-receptor pair to visualise. Ignored when both \code{ligand} and
\code{receptor} are supplied. Default \code{1} (top-ranked pair).}

\item{ligand}{Character. Ligand gene symbol. When both \code{ligand} and
\code{receptor} are provided the matching LR pair is located automatically
and \code{index} is ignored. Must be supplied together with
\code{receptor}.}

\item{receptor}{Character. Receptor gene symbol. Must be supplied together
with \code{ligand}.}

\item{lr_pair}{Character. Column name in the CCI scores data frame
corresponding to the ligand-receptor pair to visualise
(e.g. \code{"CXCL12_CXCR4"}).}
}
\value{
A \code{Heatmap} object.
}
\description{
Generic function. Reshapes the CCI data frame into a receiver-by-sender
cell-type matrix for one selected LR pair and draws a clustered heatmap.
}
\section{Methods (by class)}{
\itemize{
\item \code{plotCCILR(blisa)}: Method for a \code{blisa} object. The LR pair is
selected by \code{index} (default 1, the top-ranked pair) unless both
\code{ligand} and \code{receptor} are supplied, in which case the matching
row is located automatically and \code{index} is ignored. Stops with an
informative error if \code{CCI_scores} is \code{NULL} or the selected LR
pair has no significant hotspots.

\item \code{plotCCILR(default)}: Method for a CCI scores data frame (e.g. the
\code{CCI_scores} slot of a \code{blisa} object). The LR pair is selected
by column name via \code{lr_pair}.

}}
\seealso{
\code{\link{plotCCI}} for an overview heatmap across all LR pairs;
\code{\link{plotCCIsummary}} for an aggregated sender-by-receiver heatmap.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotHotspots.R
\name{plotHotspots}
\alias{plotHotspots}
\alias{plotHotspots.blisa}
\title{Spatial hotspot map for one ligand-receptor pair}
\usage{
plotHotspots(x, ...)

\method{plotHotspots}{blisa}(
  x,
  index = 1,
  ligand = NULL,
  receptor = NULL,
  log_pval = TRUE,
  p_cutoff = NULL,
  ...
)
}
\arguments{
\item{x}{A \code{blisa} object.}

\item{...}{Additional arguments passed to the method.}

\item{index}{Integer. Row index into \code{LR_results} selecting the
ligand-receptor pair to visualise. Ignored when both \code{ligand} and
\code{receptor} are supplied. Default \code{1} (top-ranked pair).}

\item{ligand}{Character. Ligand gene symbol. When both \code{ligand} and
\code{receptor} are provided the matching LR pair is located automatically
and \code{index} is ignored. Must be supplied together with
\code{receptor}.}

\item{receptor}{Character. Receptor gene symbol. Must be supplied together
with \code{ligand}.}

\item{log_pval}{Logical. If \code{TRUE} (default), colour significant bins
by -log10(p-value). If \code{FALSE}, use 1 - p-value.}

\item{p_cutoff}{Numeric or \code{NULL}. When \code{NULL} (default), the
pre-computed hotspot bins stored in the \code{blisa} object are used,
reflecting the \code{p_cutoff} and High-High quadrant classification
applied during \code{\link{blisa}}. When a numeric value is supplied,
bins are re-defined on the fly as those with \code{all_pval <= p_cutoff}
and quadrant label \code{"High-High"} (from the stored
\code{all_quadrant}), giving an exact re-threshold consistent with the
original classification.}
}
\value{
A \code{ggplot} object.
}
\description{
Generic function. Plots each bin coloured by significance status: empty,
non-significant, or significant hotspot (continuous gradient of
-log10 p-value or 1 - p-value).
}
\section{Methods (by class)}{
\itemize{
\item \code{plotHotspots(blisa)}: Method for a \code{blisa} object.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterLRpairs.R
\name{filterLRpairs}
\alias{filterLRpairs}
\title{Filter ligand-receptor pairs by expression threshold}
\usage{
filterLRpairs(
  counts,
  min_ligand = 10,
  min_receptor = 10,
  LR_df = NULL,
  species = c("human", "mouse")
)
}
\arguments{
\item{counts}{Gene-by-bin count matrix (dense or sparse). Row names must be
gene symbols.}

\item{min_ligand}{Numeric. Minimum count threshold for ligand genes. At least
one bin must meet or exceed this value. Default 10.}

\item{min_receptor}{Numeric. Minimum count threshold for receptor genes. At
least one bin must meet or exceed this value. Default 10.}

\item{LR_df}{Data frame of ligand-receptor pairs with columns
\code{ligand.symbol} and \code{receptor.symbol} (comma-separated gene
symbols for multi-subunit complexes). When \code{NULL}, the CellChatDB for
the chosen \code{species} is downloaded automatically.}

\item{species}{Character. Which CellChatDB to download when \code{LR_df} is
\code{NULL}. One of \code{"human"} (default) or \code{"mouse"}.}
}
\value{
A subset of \code{LR_df} containing only pairs that pass the
expression thresholds for both ligand and receptor.
}
\description{
Retains only LR pairs where at least one bin/spot has counts at or above
\code{min_ligand} for every ligand subunit and \code{min_receptor} for every
receptor subunit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blisa-class.R
\name{new_blisa}
\alias{new_blisa}
\title{Create a blisa object}
\usage{
new_blisa(LR_results, bins, spatial_weights, CCI_scores = NULL)
}
\arguments{
\item{LR_results}{Data frame of BLISA results, one row per ligand-receptor
pair.}

\item{bins}{An \code{sf} object of hexagonal bin polygons.}

\item{spatial_weights}{Spatial weights list from
\code{\link{computeSpatialWeights}}.}

\item{CCI_scores}{Wide data frame of cell-cell interaction scores from
\code{\link{runCCI}}, or \code{NULL} if CCI was not computed.}
}
\value{
An object of class \code{blisa}.
}
\description{
Constructor for the \code{blisa} S3 class, which stores the full output of
\code{\link{blisa}}.
}
\keyword{internal}
